'*******************************************************************************************************
'          DF PLAYER Ver.1.1 SW UART - DEMO PRIMJER 5 RTC + I2C LCD - zaustavlja Do-Loop petlju

' Ovo je DEMO program za ucenje koristenja naredbi DF Playera u Ver. 1.1

' Za I2C LCD koristi se driver "i2c_lcd_module.inc" , treba ga u program ukljuciti naredbom $include

' Taj driver koristi biblioteku "lcd_i2c_v2.lib" koju treba staviti u LIB direktorij BASCOM-a AVR

' I2C LCD se spaja na SDA=PD.6, SLC=PD.7, a dodatno slobodni za upotrebu su PB.4;PB.5 i PORTC

' Koristi ATmega328P (moze i ATmega8), radni takt je 16MHz (moze i 8MHz)

' VAZNO! Koristi softverski UART, sa J2 spoji PB.0 i TX, sa J1 spoji PD.1 i Key1

' Ovaj primjer za realno vrijeme koristi RTC DS3231 spojen na SCL = PD.7 i SDA = PD.6

' Za RTC podrsku koristi se biblioteka "DS3231$SE.sub" preuzeta sa internetske stranice Svet Elektronike

' Za namjestanje parametara RTC-a koristi se biblioteka "rtc_set.inc" i naredba Ds3231$set

' Podesavanje RTC-a izvrsava se samo nakon reseta drzanjem tipke Enter$ = KEY2 u trajanju 5s

' Namjestanje vremena i datuma radi se tipkama Key0 za Up i Key1 za Down, a prihvat tipkom Key2

' Ovaj primjer probno koristi TIMER0 interapt za invertiranje izlaza PD.5 svake sekunde

' Selekcija opcija 0 do 9 u Do-Loop petlji radi se tipkama Key0 i Key1, a prihvat opcije tipkom Key2

' Select=0 , izgovara poruku predstavljanja govornika " Ja sam ..."

' Select=1 , izgovara brojcane iznose niza od 10 Single varijabli

' Select=2 , izgovara redom brojeve od 1 do 10

' Select=3 , izgovara redom brojeve od 10 do 1

' Select=4 , izgovara sobnu temperaturu (temperaturu chipa DS3231)

' Select=5 , izgovara dan, vrijeme, datum, mjesec i godinu iz RTC-a

' Select=6 , izgovara generirani iznos temperature iz Single varijable Y

' Select=7 , demo reprodukcija mp3 Gibonni "Vrime da se pomirim sa svitom"

' Select=8 , demo reprodukcija mp3 Oliver "Dva put san umra" - Instrumentalna prerada

' Select=9 , demo reprodukcija mp3 "Shallow"

' Tipka Key3 - prekida reprodukciju trenutnog mp3 zapisa (ne i niza mp3 zapisa)

' Tipka Key4 - ukljucuje LED potprogramom Urgent

' Izabran je izgovor na hrvatskom jeziku, stavi SD karticu sa MP3 direktorijem za hrvatski jezik
'-------------------------------------------------------------------------------------------------------
'Za vrijeme reprodukcije zaustavlja prolaz kroz Do-Loop petlju

'Softverski UART: za TX je izabran PB.0 , za Busy = PB.1 ; izbori po zelji

'Napomena:Za vrijeme slanja podatka na DF Player onemoguceni su svi interapti

'Sa $include "dfplayer1_1.inc" u program ukljuciti slijedece naredbe za DF player:

'Dfplayer Command, Parameter - naredba slanja komande i opciono parametra na DF player

'najcesca Dfplayer naredba je: Dfplayer Mp3, x  (x je brojcani naziv mp3 zapisa u direktoriju MP3)

'Say Lab_message  -  naredba slanja poruke ili mp3 muzike na DF Player

'Say_number x - izgovor broja iz Single varijable x (opseg broja je od -9999.9 do 9999.9)

'Say_temperature x - izgovor temperature iz Single varijable x i mjerne jedinice za temperaturu

'Say_day - izgovor dana u tjednju iz Byte varijable Day

'Say_date - izgovor datuma, mjeseca i godine (hrvatski ili englesni standard izgovora)

'iz Byte varijabli Dte, Month i Year (brojevi u dekatskom obliku)

'Say_time - izgovor vremena HH:MM iz Byte varijabli Hour i Minute(sadrzaj u dekatskom obliku)

'Byte varijable Day, Hour, Minute, Dte, Month i Year dimenzionirane su u biblioteci "dfplayer1_1.inc"

'Sve navedene naredbe za DF player mogu se koristiti izvan ili unutar Do-Loop petlje

'Pocetna glasnoca je 28, a ekvilajzer je na Normal

'VAZNO! Naredbe DF Playera u Ver. 1.1 ne koriste interapte, ali u ovom primjeru koristi se TIMER0

'za invertiranje izlaza PD.5 svake sekunde, pa su za ostalu upotrebu slobodni TIMER1 i TIMER2

'Radni takt moze biti ili 16MHz ili 8MHz zbog TIMER0 interapta (izabrano 16MHz)

'Autor: Mladen Cempuh ; E-mail: cempuh.mladen@gmail.com

'Datum: 12.05.2021.
'****************************************************************************************************
$regfile = "m328pdef.dat"                                   ' za ATmega328P
$crystal = 16000000                                         ' izabran radni takt 16MHz (moze biti i 8MHz)
$swstack = 50
$hwstack = 50                                               ' za DF Player Ver.1.2  HW Stack treba biti >= 50, po potrebi povecaj
$framesize = 64

'---------------------------------------------------------
'DEFINIRANJE ULAZA,IZLAZA I POCETNIH STANJA PINOVA PORTOVA
'DDRn   PORTn
' 0       0   ==> Input 3-state
' 0       1   ==> Input
' 1       0   ==> Output=0
' 1       1   ==> Output=1

  Ddrb = &B11111101                                         ' izlaz PB.0 za TX, ulaz PB.1 za Busy, PB.2 izlaz za LED, PB3 izlaz za rasvjetu LCD-a
 Portb = &B11001011                                         ' Izlazi za LCD: DB7=PB.4;DB6=PB.5;DB5=PC.2;DB4=PC.3;E=PC.4;RS=PC.5
  Ddrc = &B11111100                                         ' PC.0 i PC.1 su slobodni ulazi
 Portc = &B00000011                                         '
  Ddrd = &B11100000                                         ' Ulazi za tipke KEY0=PD.0;KEY1=PD.1;KEY2=PD.2;KEY3=PD.3;KEY4=PD.4
 Portd = &B11111111                                         ' PD5,PD6 i PD7 su slobodni za koristenje (startno postavljeni kao izlazi sa stanjem 1)

'------------------------------------------------------------
Up$ Alias Pind.0                                            ' ulaz za tipku Up (+) za RTC

Down$ Alias Pinb.0                                          ' ulaz za tipku Down (-) za RTC, opcija samo ako je konstanta Down_key=1

Enter$ Alias Pind.2                                         ' ulaz za tipku Enter za RTC

Const Down_key = 1                                          ' Konstanta Down_key=0 --> ne koristi se tipka Down$ ,a ako je 1 koristi se

'------------------------------------------------------------
'        Inicijalizacija I2C komunikacije i I2C LCD
'------------------------------------------------------------
 Config Scl = Portd.7                                       ' izlaz SCL

 Config Sda = Portd.6                                       ' izlaz SDA

 I2cinit                                                    ' inicijaliziraj I2C komunikaciju

 Const Pcf8574_lcd = &H4E                                   'default adresa za PCF8574T je &H4E (A0=1,A1=1,A2=1),
                                                             'a za PCF8574AT (A0=1,A1=1,A2=1) je &H7E
                                                             'po potrebi redefiniraj adresu za PCF chip
 Config Lcd = 16 * 2                                        'izbor LCD displeja

'podrzani displeji su 16 * 2, 16 * 4 , 20 * 2, 20 * 4 , 16 * 1a
'Ako nije ukljucena naredba Config Lcd tada je izabran 16 * 2

$include "i2c_lcd_module.inc"                               ' driver za rad LCD-a preko I2C komunikacije

Cls

'-------------------------------------------------------------
$include "DS3231$SE.sub"                                    ' biblioteka za RTC DS3231, preuzeto sa linka Sveta Elektronike

$include "rtc_set.inc"                                      ' biblioteka za namjestanje vremena RTC-a
'-------------------------------------------------------------
 Lcd "ENTER 5s FOR RTC"                                     ' drzi 5s tipku KEY2 (Enter) za ulazak u namjestanje RTC-a

 Wait 1

 Ds3231$write_control &B00000000                            ' SQW=1Hz, Disable INT, Disable Alarm1,Alarm2, Disable Temperature

 Ds3231$set                                                 ' naredba za namjestanje RTC-a

'----------------------------------------------------------------------------
'Definiraj izlaz TX i ulaz Busy, a zatim ukljuci biblioteku "dfplayer1_1.inc"

'Konfiguracija Software UART-a za DF Player:

Open "comb.0:9600,8,n,1" For Output As #1                   'PB.0 za TX , RX se ne koristi

                                                              'Baud=9600, 8 Bits, No parity, 1 Stop bit
'Definiraj ulaz Busy:

Busy Alias Pinb.1

Config Busy = Input                                         ' ulaz za Busy sa DF Playera, 1=slobodan , 0=zauzet

$include "dfplayer1_1.inc"                                  ' biblioteka za DF Player u Ver.1.1 (softverski UART)

Language = 1                                                ' 1 = hrvatski , 0 = engleski; izabran je hrvatski
'-----------------------------------------------------------------------------------------------------------------
'Definiranje 5 tipki spojenih na PORTD i GND

Key0 Alias Pind.0
Key1 Alias Pind.1
Key2 Alias Pind.2
Key3 Alias Pind.3
Key4 Alias Pind.4


Led Alias Portb.2

'Config Led = Output                                         ' vec je definirano  kao izlaz sa DDRB
'--------------------------------------------------------------------------------------------------

  Timer0_data Alias 63                                      'konstanta za definiranje TIMER0 interapta 100us na 16MHz ili 200ms na 8 MHz

                                                             'vrijednost konstante odredjena mjerenjem vremena izmedju dva interapta

Config Timer0 = Timer , Prescale = 8                        'za fosc=16MHz i Prescale = 8, Tclock= 0,5us
                                                              'za fosc=8MHz i Prescale = 8, Tclock= 1us

Timer0 = Timer0_data                                        'brojac TIMER0 napuni pocetnom vrijednosti za Tint

On Timer0 Timer0_isr                                        'Timer0_isr je za obradu TIMER0 interapta

Dim Counter_for_1ms As Byte                                 'brojac za generirane ISR-a od 1ms

#if _xtal = 16000000                                        'ako je $crystal=16000000 tada je

    Factor_for_1ms Alias 9                                  'Factor_for_1ms = (1000-Tint)/Tint = 9

#endif

#if _xtal = 8000000                                         'ako je $crystal=8000000 tada je

    Factor_for_1ms Alias 4                                  'Factor_for_1ms = (1000-Tint)/Tint = 4

#endif

Factor_for_1s Alias 1000                                    '1000 x 1ms = 1s

Dim Counter_for_1s As Word                                  'brojac za generiranje dijela ISR-a od 1s

Counter_for_1s = Factor_for_1s                              'napuni ga pocetnim iznosom 1000

Enable Timer0                                               'omoguci TIMER0 prekid

Enable Interrupts                                           'omoguci sve dozvoljene prekide

'-------------------------------------------------------------------------------------------

'  Dfplayer Volume , 31                                      'pocetna glasnoca je 28,opciono promjeni glasnocu npr. na 31 --> maksimum
'  Waitms 200                                                'pauza za izvrsenje naredbe
'--------------------------------------------------------------------------------------------------------------------------

  Dim X(10) As Single , I As Byte , Y As Single , W As Word , D As Byte , Predznak As Byte , C As Byte , Sel As Byte

  Dim Room_temp As Single , Temp_read As Word

'Niz Single varijabli za izgovor:

  X(1) = -9111.9 : X(2) = 1.9 : X(3) = 1041 : X(4) = 0 : X(5) = -8.5 : X(6) = 5370.1 : X(7) = -1000 : X(8) = -218.6 : X(9) = 205 : X(10) = -6101.7


 Locate 1 , 1 : Lcd "DF Player Ver1.1"

 Locate 2 , 1 : Lcd "Demo primjer 5"

 Wait 2

 Cls


Do

   Ds3231$read                                              'ucitavanje sec, min, hour, day, date, month, year

   Locate 1 , 1                                             ' u prvom redu LCD-a od pozicije 1
   Lcd_dan_hr                                               ' ispis dana u tjednu
   Lcd " ";
   Lcd_hh_mm_ss                                             ' i vremena u formatu HH:MM:SS

'U slijedece varijable upisi dekatske iznosi za vrijeme HH:MM, dan u tjednu, datum, mjesec i godinu (a ne BCD)
'Napomena:Ako koristis RTC (npr.DS3231) tada prije upisa BCD iznose pretvori naredbom Makebcd u dekatske iznose

   Hour = Makedec(ds3231$hour)
   Minute = Makedec(ds3231$min)
   Day = Ds3231$day
   Dte = Makedec(ds3231$dte)
   Month = Makedec(ds3231$mth)
   Year = Makedec(ds3231$yr)


   Locate 2 , 1 : Lcd "Select(0-9) = " ; Sel ; " "

   Debounce Key0 , 0 , Sel_incr , Sub                       ' tipkom Key0 inkrementiraj varijablu Sel za Select (0-9)

   Debounce Key1 , 0 , Sel_decr , Sub                       ' tipkom Key1 dekrementiraj varijablu Sel za Select (0-9)

   Debounce Key2 , 0 , Enter , Sub                          ' tipka Key2 je Enter za izvrsenje izabrane opcije sa varijablom Sel tj. Select(0-9)


   Gosub Urgent                                             ' potprogram kojim se nadziru vazna stanja neovisno o reprodukciji

Loop

Sel_incr:

   Incr Sel

   If Sel > 9 Then Sel = 0                                  ' kruzno inkrementiranje varijable Sel od 0 do 9

Return

Sel_decr:

   Decr Sel

   If Sel = 255 Then Sel = 9                                ' kruzno dekrementiranje varijable Sel od 9 do 0

Return


Enter:

  Select Case Sel

  Case 0 : Gosub Tko_si_ti                                  ' izgovara poruku predstavljanja govornika " Ja sam ..."

  Case 1 : Gosub Say_numbers                                ' izgovara brojcane iznose niza od 10 Single varijabli

  Case 2 : Gosub Numbers_1to10                              ' izgovara redom brojeve od 1 do 10 naredbom Say Lab_message

  Case 3 : Gosub Numbers_10to1                              ' izgovara redom brojeve od 10 do 1 naredbom Say Lab_message

  Case 4 : Gosub Speak_room_temperature                     ' izgovara sobnu temperaturu (temperaturu chipa DS3231)

  Case 5 : Gosub Speak_day_time_date                        ' izgovara tocan dan, tocno vrijeme, tocan datum, mjesec i godinu iz RTC-a

  Case 6 : Gosub Random_temperature                         ' izgovara generirani iznos temperature iz Single varijable Y

  Case 7 : Gosub Gibonni_mp3                                ' demo reprodudukcija mp3 Gibonni "Vrime da se pomirim sa svitom"

  Case 8 : Gosub Oliver_mp3                                 ' demo reprodukcija mp3 Oliver "Dva put san umra" - Instrumentalna prerada

  Case 9 : Gosub Shallow_mp3                                ' demo reprodukcija mp3 "Shallow"

  End Select

  Cls

Return

'----------------------------------------------------------
'          TIMER0 - INTERAPT SERVISNA RUTINA
'----------------------------------------------------------
Timer0_isr:

'izvrsava se svakih 100us na 16MHz ili svakih 200us na 8MHz

Timer0 = Timer0_data                                        'napuni brojac TIMER0 pocetnom vrijednosti

'-----------------------------------------------------
'           ZA ISR PERIODE 1ms
'-----------------------------------------------------

   If Counter_for_1ms > 0 Then
      Decr Counter_for_1ms
      Goto End_isr_1ms
   End If

'----------------------------------------------------
'          EVERY 1ms
'----------------------------------------------------
'izvrsava se svake 1ms

 Counter_for_1ms = Factor_for_1ms

 Decr Counter_for_1s

 If Counter_for_1s > 0 Then Goto End_isr_1ms

'----------------------------------------------------
'          EVERY 1s
'----------------------------------------------------
'izvrsava se svake sekunde

 Counter_for_1s = Factor_for_1s

 Toggle Portd.5                                             'invertiraj stanje izlaza PD.5

'----------------------------------------------------

End_isr_1ms:

 Return

'---------------------------------------------------------

Say_numbers:

   Cls

   Lcd "Say X(1)...X(10)"

   For I = 1 To 10                                          ' ispis na LCD i izgovor brojcanih iznosa niza od 10 Single varijabli

   Locate 2 , 1 : Lcd "X(" ; I ; ")=" ; Fusing(x(i) , "####.#") ; "     "       ' reci brojcani iznos Single varijable X(i)

   Say_number X(i)                                          ' reci brojcani iznos Single varijable X(i)

   Say Pause_2s                                             ' tisina 2s za vrijeme koje se izvrsava potprogram Urgent
                                                            ' za razliku od npr. Wait 2 kad se nista ne izvrsava
   Next I

Return

'-----------------------------------------------------------
Numbers_1to10:

  Cls

  Lcd "Broji 1 ... 10"

  Say Broji_1do10                                           ' izgovara redom brojeve od 1 do 10

Return

'-----------------------------------------------------------
Numbers_10to1:

  Cls

  Lcd "Broji 10 ... 1"

  Say Broji_10do1                                           ' izgovara redom brojeve od 10 do 1

Return
'-----------------------------------------------------------
Speak_day_time_date:

   Locate 2 , 1 : Lcd "Naredba:Say_day "

   Say Today_is                                             ' izgovara "Danas je"

   Say_day                                                  ' reci koji je dan

   Locate 2 , 1 : Lcd "Naredba:Say_time"

   Say Pause_1s                                             ' tisina 1s za vrijeme koje se izvrsava potprogram Urgent

   Say The_time_is                                          ' izgovara "Sada je"

   Say_time                                                 ' reci vrijeme HH:MM

   Cls

   Lcd_dd_mm_20yy                                           ' ispisi datum, mjesec i godinu

   Locate 2 , 1 : Lcd "Naredba:Say_date"

   Say Pause_1s                                             ' tisina 1s za vrijeme koje se izvrsava potprogram Urgent

   Say Date_is                                              ' izgovara "Datum je"

   Say_date                                                 ' reci datum, mjesec i godinu (EU nacin za hrvatski ili UK/US nacin za engleski)

   Return

'-----------------------------------------------------------
Speak_room_temperature:

 Ds3231$read_temp Temp_read                                 ' ucitaj temperaturu iz chipa DS3231

 Room_temp = Temp_read / 256                                'izracunaj temperaturu u stupnjevima Celzija

 Cls

 Lcd "Temperatura sobe"

 Locate 2 , 1 : Lcd "t=" ; Fusing(room_temp , "##.#") ; Chr(223) ; "C  "       ' ispis temperature chipa DS3231 zaokruzen na jednu decimalu

 Say Temperature_is

 Say_temperature Room_temp                                  ' izgovara pozitivnu temperaturu iz DS3231 RTC chipa (jedna decimala)

Return
'-----------------------------------------------------------
Gibonni_mp3:

  Cls

  Lcd "Muzika: Gibonni"

  Say Gibonni                                               ' demo reprodukcija mp3 Gibonni "Vrime da se pomirim sa svitom"

Return
'-----------------------------------------------------------
Oliver_mp3:

  Cls

  Lcd "Muzika: Oliver"

  Say Oliver                                                ' demo reprodukcija mp3 Oliver "Dvaput san umra" - Instrumentalna prerada

Return

'-----------------------------------------------------------
Shallow_mp3:

  Cls

  Lcd "Muzika: Shallow"                                     ' demo reprodukcija mp3 "Shallow"

  Say Shallow

Return

'-----------------------------------------------------------
Tko_si_ti:

  Cls

  Lcd "Tko si ti ?"

  Say Ja_sam                                                ' izgovara poruku predstavljanja govornika: "Ja sam ..."

Return

'------------------------------------------------------------------
Random_temperature:

' Generiranje nekog broja u Single varijabli Y za iznos temperature

    W = Rnd(99) : D = Rnd(9)
    Predznak = Rnd(9)
    C = Rnd(9)
    Y = 0
    If C >= 5 Then
    Y = 0.1 * D
    End If

    Y = Y + W
    If Predznak <= 5 Then
    Y = -1 * Y
    End If

   Cls

   Lcd "Temperatura"

   Locate 2 , 1 : Lcd "t=" ; Fusing(y , "##.#") ; Chr(223) ; "C  "

   Say_temperature Y                                        'izgovara generiran broj za temperaturu i kaze ispravno jedinicu za Celzij

Return
'-----------------------------------------------------------
Urgent:

Debounce Key3 , 0 , Stopping_play , Sub                     ' pritiskom na Key3 prekida reprodukciju trenutnog mp3 zapisa (ne i niza mp3 zapisa)


   If Key4 = 0 Then                                         ' ako je pritisnuta tipka Key4 ukljuci Led

      Set Led

   Else

    Reset Led                                               ' u protivnom iskljuci Led

   End If

Return

Stopping_play:

   Dfplayer Stop_play , 0                                   ' opciono iza Stop_play naredbe stavi  Waitms 200 , ovdje nije neophodno

   Locate 2 , 1 : Lcd "Prekini Play    "

'   Waitms 200                                               ' jer iza naredbe Stop_play ne slijedi odmah neka druga Dfplayer naredba

Return

End                                                         ' kraj programa

$include "MP3.bas"                                          'datoteka u programu koristenih mp3 zapisa