'--------------------------------------------------------------------------------------------------------------
' Biblioteka za DF Player 2.2 preko HW UART - Za vrijeme reprodukcije ne zaustavlja izvrsavanja Do-Loop petlje

' Autor: Mladen Cempuh , Datum: 19.09.2021.  ; E-mail: cempuh.mladen@gmail.com
'--------------------------------------------------------------------------------------------------------------

'Napomena: Umetnuti u glavni program naredbom $include "dfplayer2si.inc"

'Podrzava izbor pravila izgovora varijablom Language = 0 --> engleski, 1 --> hrvatski , 2 --> slovenski

'Napomena: Za slanje podataka na DF Player koristi hardverski UART i TX je PD.1

'          Za vrijeme slanja podatka na DF Player dozvoljeni su prekidi.

'          Koristi TIMER0 prekidnu rutinu i Df buffer

'          Ne zaustavlja izvrsavanje Do-Loop petlje

'          Koristi se TF card(micro SD), glasnoca je 28, ekvilajzer = 0 --> Normal

'          Izlaz Error je za prijavu greske prepunjenosti Df buffera pomocu Error Led diode

'          LCD displej je obavezan

'          Radni takt mikrokontrolera moze biti 8MHz ili 16 MHz

'Naredbe drivera za DF Player:

'Prije Do_loop petlje: 'Dfplayer Command, Parameter - naredba slanja komande i opciono parametra na DF player

'Unutar Do_loop petlje:

'Dfbuffer_enable - naredba se mora staviti samo jednom iza naredbe Do , a prije ostalih naredbi

'                  za DF player i omogucava pravilan rad Df_buffera

'Dfbuffer x  -  naredba za upis u Df buffer (x je broj naziva mp3 zapisa u MP3 direktoriju)

'Dfstop - naredba brise sadrzaj Df buffera i zaustavlja reprodukciju

'Say Lab_message  -  naredba slanja poruke u Df buffer , poruka se nalazi npr. na labeli Lab_message

'Say_number x - izgovor broja iz Single varijable x (opseg broja je od -9999.9 do 9999.9)

'Say_temperature x - izgovor temperature iz Single varijable x (od -9999.9 do 9999.9) i izgovor mjerne jedinice

'Say_day - izgovor dana u tjednu iz Byte varijable Day

'Say_time - izgovor vremena (HH:MM) iz Byte varijabli Hour i Minute

'Say_date - izgovor datuma, mjeseca i godine iz Byte varijabli Dte, Month i Year

'Dodatne mogucnosti:

'                   Koristenje 32 bitnog brojaca Millis za vrijeme u ms od prorade programa

'                   Naredba Wait_ms za tocnu pauzu u ms pomocu TIMER0 prekidne rutine

'                   Linkovi: Every_1ms: , Every_10ms: , Every_100ms: , Every_1s:

'----------------------------------------------------------------------------------------------------------------
'Upravljacke komande Dfplayer naredbe:

'Play_next Alias &H01                                        'reproduciraj slijedecu datoteku

'Play_prev Alias &H02                                        'reproduciraj prethodnu datoteku

'Play Alias &H03                                             'reproduciraj datoteku iz root direktorija (problem imenovanja!)

Volup Alias &H04                                            'uvecaj glasnocu za 1

Voldown Alias &H05                                          'smanji glasnocu za 1

Volume Alias &H06                                           'namjesti glasnocu na 0 do 31

Equlizer Alias &H07                                         'namjesti ekvilajzer , 0 = Normal

'Playmode Alias &H08                                         'izbor nacina ponavljanja reprodukcije

Source Alias &H09                                           'izaberi izvor zapisa datoteka, 1 =TF card = micro SD card

Standby Alias &H0A                                          'postavi  u Standby (Low Power)

Wakeup Alias &H0B                                           'Wakeup - izadji iz Standby moda u Normal

Resetmodule Alias &H0C                                      'software-ski reset DF Playera

'Playback Alias &H0D                                         'nastavi pauziran zapis

'Pause Alias &H0E                                            'pauziraj trenutnu reprodukciju

'Specfolderplay Alias &H0F                                   'izaberi folder za reprodukciju

'Voladjust Alias &H10                                        ' namjestanje glasnoce (drugi nacin)

'Repeatplay Alias &H11                                       'toggle ponavljanja reprodukcije svih datoteka 1=Start repeat, 0=Stop repeat

Mp3 Alias &H12                                              'reprodukcija izabrane mp3 datoteke (1 do 9999) iz MP3 foldera

'Start_advert Alias &H13                                     'prekida trenutnu reprodukciju i ubacuje reklamu iz ADVERT foldera

'Stop_advert Alias &H15                                      'zaustavlja reprodukciju reklame iz ADVERT foldera i nastavlja prethodno prekinutu reprodukciju

Stop_play Alias &H16                                        'zaustavlja reprodukciju trenutno reproducirane datoteke

'Repeat_folder Alias &H17                                    'ponovi reprodukciju izabranog foldera (1-99)

'Random_play Alias &H18                                      'random reprodukcija sa medija

'Repeat_play Alias &H19                                      'ponovna reprodukcija pokrenute datoteke

Amplifier_off Alias &H1A                                    'tiho iskljucivanje pojacala
'-------------------------------------------------------------------------------------------------------
'Hardverski UART za TX (PD.1):

$baud = 9600                                                ' baud rate 9600 bits/s, 8 bits, no parity, 1 stop bit

'Opcija koristenja izlaznog serijskog buffera:

'Config Serialout = Buffered , Size = 12                     ' buffer za serijski RS232 TX izlaz

'Enable Interrupts                                           ' tada je neophodno omoguciti  sve interapte (zbog UDRE interapta koji se koristi)
                                                              ' ako se ne koristi buffer nije potrebno omoguciti interapte
'---------------------------------------------------------------------------------------------------------------------
Tf_card Alias 1                                             ' radi sa Micro SD card (TF card)


Config Error = Output                                       ' Odabrani pin Error je izlazni pin
'-------------------------------------------------------------------------------------------------------

'Definiranje varijabli za Df_buffer:

Dim Mp3_file As Word

Dim Df_buffer(dfbuffer_len) As Word                         ' kreiraj Words Df_buffer za DF Player

Dim Write_index As Byte , Read_index As Byte                ' indeksi za upis u Df_buffer i za citanje iz Df_buffera

Write_index = 1 : Read_index = 1                            ' pocetno su indeks upisa i indeks citanja Df_buffera postavljeni na poziciju 1

Dim Df_flag As Bit                                          ' flag za zabranu upisa u Df_buffer, 0=dozvoli , 1=zabrani , startno je 0

Dim Clear_index As Byte                                     ' za naredbu Dfclear

Dim _$cifra As Byte , Language As Byte                      ' za naredbu Say_temperature x

Dim Hour As Byte , Minute As Byte , Day As Byte             ' Byte varijable za naredbu Say_time  i Say_day

Dim Dte As Byte , Month As Byte , Year As Byte              ' Byte varijable za naredbu Say_date

'-------------------------------------------------------------------------------------------------------------------------------------------------
Config Submode = New

Goto Setup_timer0

'---------------------------------------------------------------
'       Naredba: Dfplayer  Command, Parameter
'---------------------------------------------------------------

Sub Dfplayer(byval Command As Byte , Byval Parameter As Word)

'Format slanja ($S ; Ver ; Len ; Cmd ; Feedback ; Para1 ; Para2 ; Checksum1 ; Checksum2 ; $O)

   Local _$s As Byte , _ver As Byte , _len As Byte , _$o As Byte , Feedback As Byte

   Local Checksum_ As Integer , Checksum1 As Byte , Checksum2 As Byte , Para1 As Byte , Para2 As Byte


   Para1 = High(parameter)

   Para2 = Low(parameter)

   _$s = &H7E                                               'Start command
   _ver = &HFF                                              'Version information
   _len = &H06                                              'Data lenght (od Version do Para2 (data_low DL)
   _$o = &HEF                                               'End command

'Izracun Checksuma:

   Feedback = 0                                             'bez feedbacka

   Checksum_ = _ver + _len
   Checksum_ = Checksum_ + Command
'  Checksum_ = Checksum_ + Feedback                         'Feedback = 0 pa nije ovaj red potreban
   Checksum_ = Checksum_ + Para1
   Checksum_ = Checksum_ + Para2
   Checksum_ = 0 - Checksum_
   Checksum1 = High(checksum_)
   Checksum2 = Low(checksum_)

'Posalji na TX:

    Print Chr(_$s) ; Chr(_ver) ; Chr(_len) ; Chr(command) ; Chr(feedback) ; Chr(para1) ; Chr(para2) ; Chr(checksum1) ; Chr(checksum2) ; Chr(_$o)

End Sub

'------------------------------------------------------------
'              Naredba: Say Lab_message
'------------------------------------------------------------
Dim Say_label As Word , Take_data As Integer                'definiranje varijabli za naredbu Say

Sub Say(bylabel Lab_message As Word)

 If Df_flag = 1 Then Goto End_say                           ' ako je Df_flag=1 naredba Say za upis poruke u Df_buffer je onemogucena

   Say_label = Lab_message

   lds _dptrl,{Say_label}                                   ' Napuni Data pointer sadrzajem argumenta Lab_message za naredbu Read
                                                            ' to je umjesto naredbe Restore kojom se Data pointer ne moze proslijediti
   LDS _dptrh,{Say_label+1}                                 ' u deklariranu subrutinu

Read_tab:

   Read Take_data                                           ' procitaj podatak sa mjesta oznacenog sadrzajem Data pointera iz Data tablice

   If Take_data <> 0 Then                                   ' 0=kraj tablice, ako nije nula tada

    Mp3_file = Take_data                                    'prebaci podatak u Word varijablu Mp3_file koja je argument naredbe Dfbuffer

    Dfbuffer Mp3_file                                       ' upisi broj mp3 zapisa u Df_buffer

   Goto Read_tab                                            'slijedece citanje, Data pointer se nakon prethodne Read naredbe automatski uvecao
                                                            'na slijedecu poziciju u tablici
   End If

End_say:

End Sub

'------------------------------------------------------------
'               Naredba: Dfbuffer x
'------------------------------------------------------------

Sub Dfbuffer(byval Mp3_num As Word)

  If Df_flag = 1 Then Goto End_dfbuffer                     'onemogucen je upis u Df_buffer u prolazu kroz Do..Loop, jer je navedeni flag setiran
                                                            'zato sto reprodukcija upisana u prethodnim Do-Loop petljama iz Df_buffera jos nije zavrsena

   Df_buffer(write_index) = Mp3_num                         ' stavi broj mp3 datoteke u Df_buffer na poziciju Write_index

   Incr Write_index                                         ' uvecaj Write_index na slijedecu poziciju

   If Write_index > Dfbuffer_len Then Goto Buffer_overload  'ako je nakon uvecanja Write_index izasao iz Df_buffera prijavi gresku:Buffer overload!

End_dfbuffer:

End Sub

Buffer_overload:

Locate 2 , 1 : Lcd "Buffer overload!"

Blink_error:

Toggle Error

Waitms 250

Goto Blink_error

'------------------------------------------------------------
'              Naredba: Dfbuffer_enable
'------------------------------------------------------------

Sub Dfbuffer_enable                                         'staviti je samo jednom u Do-Loop petlju, odmah iza naredbe Do

   If Write_index > 1 Then

      Set Df_flag                                           ' omoguceno izvrsavanje u interaptu rutine Read_buffer, a onemogucena naredba Dfbuffer

      Else

      Reset Df_flag                                         ' omogucena naredba Dfbuffer, a onemogucena interapt rutina Read_buffer

   End If

End Sub

'------------------------------------------------------------
'         Naredba: Dfstop
'------------------------------------------------------------

Sub Dfstop

  For Clear_index = 1 To Dfbuffer_len                       ' obrisi DF buffer

   Df_buffer(clear_index) = 0

  Next Clear_index

  Write_index = 1                                           'postavi indeks za poziciju upisa na pocetak DF buffera

  Read_index = 1                                            'postavi indeks za poziciju citanja na pocetak DF buffera

  Dfplayer Stop_play , 0                                    'zaustavi trenutnu reprodukciju

  Time_ms = 200

  Wait_ms                                                   'cekaj 200ms

End Sub

'-----------------------------------------------------------------------------
' Subrutina za citanje sadrzaja Df_buffera i slanje na DF player
'-----------------------------------------------------------------------------

Read_buffer:

If Busy = 0 Then Return                                     ' Busy=0,povrat iz subrutine,nije zavrsena reprodukcija prethodno izabranog mp3 zapisa

  Mp3_file = Df_buffer(read_index)                          ' uzmi podatak iz Df_buffera sa pozicije Read_index i stavi u Mp3_file

  Df_buffer(read_index) = 0                                 'upisi 0 u prethodno ocitanu poziciju u Df_bufferu

  If Mp3_file = 0 Then                                      'ako je podatak uzet iz Df_buffera 0, tada

    Write_index = 1                                         ' postavi oba indeksa na pocetak Df_buffera

    Read_index = 1

    Return                                                  'vrati se iz Read_buffer subrutine

  End If

  Dfplayer Mp3 , Mp3_file                                   'posalji u DF Player naredbu Mp3 i podatak za reprodukciju mp3 datoteke uzete iz Df_buffera

  Incr Read_index                                           ' uvecaj za 1 index citanja za slijedecu poziciju

Return                                                      'povrat iz Read_buffer subrutine

'----------------------------------------------------------------------------
'                    DF Player naredba : Say_number x
'----------------------------------------------------------------------------
'Say_number x  -  naredba izgovora broja iz Single varijable x

'Dozvoljeni opseg broja je od -9999.9 do 9999.9

Dim Num_string As String * 7 , Ar(8) As Byte , _$index As Byte , First_digit As Byte       ' definiranje varijabli za naredbu Say_number

Sub Say_number(byval Number_for_say As Single)

 If Df_flag = 1 Then Goto End_of_speak                      ' ako je Df_flag=1 naredba Say_number za upis izgovora broja u Df_buffer je onemogucena

   Num_string = Fusing(number_for_say , "####.#")           'pretvori u format 4 cjelobrojne cifre i jedna decimalna cifra

   For _$index = 1 To 8                                     'obrisi od array varijable od Ar(1) do Ar(8)

   Ar(_$index) = 0

   Next _$index

   Str2digits Num_string , Ar(1)                            'vidi znacenje naredbe Str2digits

   First_digit = 7                                          ' moguce mjesto prve cifre broja je u Ar(7)

   For _$index = 5 To 8                                     'pretrazi od Ar(5) do Ar(8)

   If Ar(_$index) = 253 Then                                ' ako nadjes broj 253, to je kod za "minus" tada

    Dfbuffer 253                                            'stavi kod za "minus" (file: 253.mp3) u Df_buffer, broj je negativan

    Decr _$index                                            ' smanji _$indeks, pointer na prvoj cifri broja

    First_digit = _$index                                   ' stavi u varijablu First_digit podatak o pointeru prve cifre broja

    Goto Test_zero                                          ' idi na Test_zero

    End If

    Next _$index

' nije naden 253 (minus) -->broj je pozitivan

Test_zero:

   For _$index = First_digit To 4 Step -1

   If Ar(_$index) <> 0 Then Goto Cifre_broja                ' cjelobrojni dio nije nula i idi na citanje broja

   Next _$index

' cjelobrojni dio je 0

   Dfbuffer 255                                             ' stavi kod za "nula" (file: 255.mp3) u Df_buffer

   If Ar(2) = 0 Then Goto End_of_speak                      'i decimala je 0, broj je 0, zavrsi izgovor broja

   Gosub Say_decimal                                        ' u protivnom reci "cijelih" i decimalu

   Goto End_of_speak                                        ' i zavrsi izgovor broja

Cifre_broja:

   If _$index = 7 Then                                      ' postoji cifra tisucice

   Mp3_file = Ar(7) * 7

   Mp3_file = Mp3_file + 1000                               'izracunaj Name_of_mp3 za cifru tisucice

   Dfbuffer Mp3_file                                        ' stavi kod mp3 izgovora tisucice broja u Df_buffer

   Decr _$index                                             ' smanji pointer _$indeks na 6

   End If


   If _$index = 6 Then                                      'ispitaj stoticu

   If Ar(6) = 0 Then Goto Index_5                           ' stotica je 0 , preskoci na slijedecu cifru

   Mp3_file = Ar(6) * 6                                     'postoji cifra stotice

   Mp3_file = Mp3_file + 100                                'izracunaj Name_of_mp3 za cifru stotice

   Dfbuffer Mp3_file                                        ' stavi kod mp3 izgovora stotice broja u Df_buffer

Index_5:

   Decr _$index                                             ' smanji pointer  _$indeks na 5

   End If

   If _$index = 5 Then                                      ' ispitaj deseticu

    If Ar(5) = 0 Then Goto Index_4                          ' desetica je 0 , preskoci na slijedecu cifru

     If Ar(5) = 1 Then

      Mp3_file = Ar(4) + 10

      Dfbuffer Mp3_file                                     ' stavi kod mp3 izgovora desetice i jedinice broja 10 do 19 u Df_buffer

      Goto Index_2                                          'preskoci zaseban izgovor jedinice

     Else

    If Language = 2 Then                                    ' za pravila slovenskog jezika

     If Ar(4) <> 0 Then                                     ' reci prvo jedinicu, a onda deseticu

       Mp3_file = Ar(4)                                     'apsolutna vrijednost broja je < 10 i >= 1

       Dfbuffer Mp3_file                                    'reci jedinicu broja

       Dfbuffer 263                                         ' reci i

       Mp3_file = Ar(5) * 10                                'apsolutna vrijednost broja je >=20

       Dfbuffer Mp3_file                                    'reci deseticu broja

       Goto Index_2                                         'preskoci zaseban izgovor jedinice

     End If

    End If

      Mp3_file = Ar(5) * 10                                 'apsolutna vrijednost broja je >=20

      Dfbuffer Mp3_file                                     ' stavi kod mp3 izgovora desetice broja u Df_buffer

     End If

Index_4:

   Decr _$index                                             ' smanji  pointer _$indeks na 4

   End If


   If _$index = 4 Then                                      ' ispitaj jedinicu

     If Ar(4) = 0 Then Goto Index_2                         ' jedinica je 0, preskoci na decimalu

      Mp3_file = Ar(4)                                      'apsolutna vrijednost broja je < 10 i >= 1

      Dfbuffer Mp3_file                                     ' stavi kod mp3 izgovora jedinice broja u Df_buffer

     End If

Index_2:

     If Ar(2) = 0 Then Goto End_of_speak                    ' ako je Ar(2)=0 nema decimale, zavrsi izgovor broja

      Gosub Say_decimal                                     ' u protivno se Df_buffer puni kodom za "cijeli" ili "cijela" ili "cijelih" i decimalom

End_of_speak:                                               ' kraj izgovora broja

End Sub
'------------------------------------------------------------------------------------------

Say_decimal:

   Select Case Ar(4)

   Case 1 : Mp3_file = 257                                  '"cijeli"

   Case 2 To 4 : Mp3_file = 256                             '"cijela"

   Case Else : Mp3_file = 254                               '"cijelih"

   End Select

   Dfbuffer Mp3_file                                        'stavi u Df_buffer kod za "cijeli" ili "cijela" ili "cijelih"

   Mp3_file = Ar(2)                                         'uzmi u Mp3_file cifru decimale

   Dfbuffer Mp3_file                                        'i stavi u Df_buffer kod za izgovor decimale

   Return

'-----------------------------------------------------------
'             Naredba: Say_temperature x
'-----------------------------------------------------------

Sub Say_temperature(byval _$temperatura As Single)

If Df_flag = 1 Then Goto End_say_temperature                'ako je Df_flag=1 naredba Say_temperature je onemogucena

  Say_number _$temperatura                                  ' izgovori brojcani iznos temperature

If Language = 0 Then                                        ' ako je Language=0 tada se radi o engleskom jeziku

  If _$temperatura <= 1 Then                                ' tada za sve iznose temperature koji su <= 1 reci "degree Celsius"

     Dfbuffer 21

  Else                                                      ' a ako je temperatura > 1 tada reci "degrees Celsius"

     Dfbuffer 22

  End If

Else                                                        ' za hrvatski ili slovenski jezik pravilno reci tri mogucnosti izgovora jedinice Celzija

  If Ar(2) = 0 Then                                         ' "stupanj Celzija" ili "stupnja Celzija" ili "stupnjeva Celzija" ,

   _$cifra = Ar(4)                                          ' a ovisno o brojcanom iznosu temperature

  Else

   _$cifra = Ar(2)

  End If

  If Ar(2) = 0 And Ar(4) = 1 And Ar(5) = 1 Then

    Dfbuffer 23                                             'stupnjeva  Celzija

  Goto Lab$_say_unit_celsius

  End If

  If Ar(2) = 0 And Ar(4) = 1 Then

   Dfbuffer 21                                              'stupanj Celzija

   Goto Lab$_say_unit_celsius

  End If

  If Ar(2) = 1 Then

   Dfbuffer 21                                              ' stupanj Celzija

    Goto Lab$_say_unit_celsius

  End If

  If _$cifra = 2 Or _$cifra = 3 Or _$cifra = 4 Then

   Dfbuffer 22                                              'stupnja Celzija

   Goto Lab$_say_unit_celsius

   End If

   Dfbuffer 23                                              'stupnjeva Celzija

Lab$_say_unit_celsius:

End If

End_say_temperature:

End Sub

'--------------------------------------------------------------------------------
'                         Naredba:Say_time
'--------------------------------------------------------------------------------
Sub Say_time()

Dim Say$hour As Word , Say$minute As Word , Df$pm As Byte

If Df_flag = 1 Then Goto End_say_time                       'ako je Df_flag=1 naredba Say_time je onemogucena

   If Language = 0 Then                                     'Engleski izgovor:

     If Hour = 0 And Minute = 0 Then                        ' ako je vrijeme 00:00

      Dfbuffer 28                                           ' reci "12 o'clock, midnight"

      Goto End_say_time                                     ' i zavrsi naredbu

     End If

     If Hour = 12 And Minute = 0 Then                       ' ako je vrijeme 12:00

      Dfbuffer 29                                           ' reci "12 o'clock, noon"

      Goto End_say_time                                     ' i zavrsi naredbu

     End If

     If Hour >= 12 Then

      Say$hour = Hour - 12                                  'izracun sati za PM

      If Say$hour = 0 Then Say$hour = 12

      Df$pm = 1                                             ' 1 za PM

     Else

      Say$hour = Hour                                       ' sati za AM

      If Say$hour = 0 Then Say$hour = 12

      Df$pm = 0                                             ' 0 za AM

     End If

      Dfbuffer Say$hour                                     ' reci sate

     If Minute <> 0 Then                                    ' ako nije 0 minuta tada

      Say$minute = Minute + 500

      Dfbuffer Say$minute                                   ' reci minute

     End If

     If Df$pm = 0 Then

      Dfbuffer 41                                           ' reci AM

     Else

      Dfbuffer 42                                           ' reci PM

     End If

     Goto End_say_time                                      ' zavrsi naredbu

   Else                                                     ' hrvatski ili slovenski izgovor:

     If Hour = 0 And Minute = 0 Then                        ' ako je vrijeme 00:00

       Dfbuffer 28                                          ' reci "ponoc"

       Goto End_say_time                                    ' i zavrsi naredbu

     End If

     If Hour = 12 And Minute = 0 Then                       ' ako je vrijeme 12:00

       Dfbuffer 29                                          ' reci "podne"

       Goto End_say_time                                    ' i zavrsi naredbu

     End If

    Say$hour = Hour + 560

    Dfbuffer Say$hour                                       ' reci sate

    Dfbuffer 263                                            'OPCIJA: reci "i"

    Say$minute = Minute + 500

    Dfbuffer Say$minute                                     ' reci minute

   End If

End_say_time:                                               ' zavrsi naredbu

End Sub

'--------------------------------------------------------------------------------
'                        Naredba:Say_day
'--------------------------------------------------------------------------------
Sub Say_day()

Dim Say$day As Word

If Df_flag = 1 Then Goto End_say_day                        'ako je Df_flag=1 naredba Say_day je onemogucena

  Say$day = Day + 30

  Dfbuffer Say$day

End_say_day:

End Sub

'--------------------------------------------------------------------------------
'                       Naredba: Say_date
'--------------------------------------------------------------------------------
Sub Say_date()

Dim Say$date As Word

If Df_flag = 1 Then Goto End_say_date                       'ako je Df_flag=1 naredba Say_date je onemogucena

  If Language = 0 Then                                      ' za engleski izgovor:

   If Dte > 12 Then

     Gosub Say_month                                        ' reci mjesec

     Gosub Say_dte                                          ' reci datum

   Else

     Gosub Say_dte                                          ' reci datum

     Gosub Say_month                                        ' reci mjesec

   End If

  Else                                                      ' za izgovor na hrvatskom ili slovenskom:

   Gosub Say_dte                                            ' reci datum

   Gosub Say_month                                          ' reci mjesec

  End If

  Say$date = Year + 400

  Dfbuffer Say$date                                         'reci godinu

  Goto End_say_date

Say_dte:

   Say$date = Dte + 300

   Dfbuffer Say$date                                        'reci datum

   Return

Say_month:

   Say$date = Month + 200

   Dfbuffer Say$date                                        ' reci mjesec

   Return

End_say_date:

End Sub

'------------------------------------------------------------
'            Setup TIMER0 interrupt
'------------------------------------------------------------
Setup_timer0:

Dim Counter_for_150ms As Byte                               'brojac za ISR rutinu Timera0 za generiranj periode od 150ms

'Matematicki: Timer0_data ~= 256 - 100*fosc(MHz)/Prescale + vrijeme spremanja na HWstack i povrata sa HWstack-a

   Timer0_data Alias 63                                     'konstanta za definiranje TIMER0 interapta 100us na 16MHz ili 200ms na 8 MHz

                                                             'vrijednost konstante odredjena mjerenjem vremena izmedju dva interapta

Config Timer0 = Timer , Prescale = 8                        'za fosc=16MHz i Prescale = 8, Tclock= 0,5us
                                                              'za fosc=8MHz i Prescale = 8, Tclock= 1us

Timer0 = Timer0_data                                        'brojac TIMER0 napuni pocetnom vrijednosti za Tint

On Timer0 Timer0_isr                                        'Timer0_isr je za obradu TIMER0 interapta

Dim Counter_for_1ms As Byte                                 'brojac za generirane ISR-a od 1ms

Dim Millis As Dword                                         '32 bitni brojac u ms od prorade programa


#if _xtal = 16000000                                        'ako je $crystal=16000000 tada je

    Factor_for_1ms Alias 9                                  'Factor_for_1ms = (1000-Tint)/Tint = 9

#endif

#if _xtal = 8000000                                         'ako je $crystal=8000000 tada je

    Factor_for_1ms Alias 4                                  'Factor_for_1ms = (1000-Tint)/Tint = 4

#endif

Dim Time_ms As Word                                         'Word varijabla za definiranje vremena interaptske pauze u ms

Dim Wait_counter_ms As Word                                 '16 bitni brojac za generiranje pauze naredbom Wait_ms

Factor_for_10ms Alias 9                                     'Factor_for_10ms iznosi 9

Dim Counter_for_10ms As Byte                                'brojac za generiranje dijela ISR-a od 10ms

Factor_for_100ms Alias 9                                    'Factor_for_100ms iznosi 9

Dim Counter_for_100ms As Byte                               'brojac za generiranje dijela ISR-a od 100ms

Factor_for_1s Alias 9                                       'Factor_for_1s iznosi 9

Dim Counter_for_1s As Byte                                  'brojac za generirane ISR-a od 1s                                '

Enable Timer0                                               'omoguci TIMER0 interapt

Goto End_of_timer0

'------------------------------------------------------------
'                   NAREDBA: Wait_ms
'           za tocnu pauzu u ms pomocu interapta od 1ms

' Izgled naredbe:
'                Time_ms=1000: Wait_ms  ' pauza 1000ms
'------------------------------------------------------------

Sub Wait_ms

   Wait_counter_ms = Time_ms

Waitms_loop:

         If Wait_counter_ms > 0 Then
         Goto Waitms_loop
         End If

End Sub

'----------------------------------------------------------
'               TIMER0 - PREKIDNA RUTINA
'----------------------------------------------------------
Timer0_isr:

'izvrsava se svakih 100us na 16MHz ili svakih 200us na 8MHz

Timer0 = Timer0_data                                        'napuni brojac TIMER0 pocetnom vrijednosti

'-----------------------------------------------------
'           ZA ISR PERIODE 1ms
'-----------------------------------------------------

   If Counter_for_1ms > 0 Then
      Decr Counter_for_1ms
      Goto End_isr_1ms
   End If

'----------------------------------------------------
'          EVERY 1ms
'----------------------------------------------------
'izvrsava se svake 1ms

 Counter_for_1ms = Factor_for_1ms

 If Wait_counter_ms > 0 Then                                ' dekrementiranje Wait_counter_ms brojaca
    Decr Wait_counter_ms                                    '  svake ms, za naredbu Wait_ms
 End If

 Incr Millis                                                ' inkrementiranje 32 bitnog brojaca ms

 Gosub Every_1ms                                            'link za kratke subrutine svake 1ms

'-----------------------------------------------------
'           ZA ISR PERIODE 10ms
'-----------------------------------------------------

   If Counter_for_10ms > 0 Then
      Decr Counter_for_10ms
      Goto End_isr_1ms
   End If

'-----------------------------------------------------
'          EVERY 10ms
'-----------------------------------------------------
'izvrsava se svakih 10ms

  Counter_for_10ms = Factor_for_10ms

  Incr Counter_for_150ms

 If Counter_for_150ms = 15 Then
   If Df_flag = 1 Then Gosub Read_buffer                    'ako je Df_flag=1 svakih 150ms izvrsi Gosub Read_buffer
   Counter_for_150ms = 0
End If

 Gosub Every_10ms                                           'link za kratke subrutine svakih 10ms

'-----------------------------------------------------
'          ZA ISR PERIODE 100ms
'-----------------------------------------------------

   If Counter_for_100ms > 0 Then
      Decr Counter_for_100ms
      Goto End_isr_1ms
   End If

'-----------------------------------------------------
'          EVERY 100ms
'-----------------------------------------------------
'izvrsava se svakih 100ms

 Counter_for_100ms = Factor_for_100ms

 Gosub Every_100ms                                          'link za kratke subrutine svakih 100ms

'-----------------------------------------------------
'          ZA ISR PERIODE 1s
'-----------------------------------------------------

   If Counter_for_1s > 0 Then
      Decr Counter_for_1s
      Goto End_isr_1ms
   End If

'-----------------------------------------------------
'          EVERY 1s
'-----------------------------------------------------
'izvrsava se svake sekunde

 Counter_for_1s = Factor_for_1s

 Gosub Every_1s                                             'link za kratke subrutine svake 1s

'-----------------------------------------------------
End_isr_1ms:

   Return                                                   'povrat iz interapta TIMERA0
'----------------------------------------------------------------------------------------

End_of_timer0:

   Wait 2                                                   ' pauza za inicijalizaciju DF Playera nakon ukljucenja napona

   Dfplayer Source , Tf_card                                ' koristi se TF card = micro SD
   Waitms 200

   Dfplayer Volume , 28                                     'pocetno namjesti glasnocu na 28 (maksimum je 31)
   Waitms 200

   Dfplayer Equlizer , 0                                    'namjesti ekvilajzer na Normal mod
   Waitms 200
'---------------------------------------------------------------------------------------------