'***********************************************************************************************************
'                    DF PLAYER Ver.2 HW UART - DEMO PRIMJER 4 RTC - ne zaustavlja Do-Loop petlju

' Ovo je DEMO program za ucenje koristenja naredbi DF Playera u Ver.2

' Zbog velicine kompajliranog programa koristi ATmega328P na 16MHz ili na 8MHz

' VAZNO! Koristi hardverski UART,sa J2 spoji PD.1 i TX, sa J1 spoji PB.0 i Key1

' Ovaj primjer za realno vrijeme koristi RTC chip DS3231 spojen na SCL = PD.7 i SDA = PD.6

' Za RTC podrsku koristi se biblioteka "DS3231$SE.sub" preuzeta sa internetske stranice Svet Elektronike

' Za namjestanje parametara RTC-a koristi se biblioteka "rtc_set.inc" i naredba Ds3231$set

' Podesavanje RTC-a izvrsava se samo nakon reseta drzanjem tipke Enter$ = KEY2 u trajanju 5s

' Namjestanje vremena i datuma radi se tipkama Key0 za Up i Key1 za Down, a prihvat tipkom Key2

' U prvom redu LCD-a ispis je dana u tjednu i vremena HH:MM:SS

' Izlaz PD.5 mijenja stanje svake sekunde pomocu linka Every_1s za prezentaciju koristenja linka Every_1s

' Selekcija opcija 0 do 9 u Do-Loop petlji radi se tipkama Key0 i Key1, a prihvat opcije tipkom Key2

' Select=0 , izgovara poruku predstavljanja govornika " Ja sam ..."

' Select=1 , izgovara brojcani iznos Single varijable X(i) , i = 1...10

' Select=2 , izgovara redom brojeve od 1 do 10

' Select=3 , izgovara redom brojeve od 10 do 1

' Select=4 , izgovara sobnu temperaturu (temperaturu chipa DS3231)

' Select=5 , izgovara dan, vrijeme, datum, mjesec i godinu iz RTC-a za trenutak pritiska na tipku Key2 (Enter)

' Select=6 , izgovara generirani iznos temperature iz Single varijable Y

' Select=7 , demo reprodukcija mp3 Gibonni "Vrime da se pomirim sa svitom"

' Select=8 , demo reprodukcija mp3 Oliver "Dva put san umra" - Instrumentalna prerada

' Select=9 , demo reprodukcija mp3 "Shallow"

' Tipka Key3 - naredbom Dfstop prekida reprodukciju trenutnog mp3 zapisa i brise Df buffer,

' te postavlja indeks I na 0. Indeks I se inkrementira prije izbora Single varijable iz niza X(1)...X(10)

' Tipka Key4 - ukljucuje LED u Do-Loop petlji

' Izabran je izgovor na hrvatskom jeziku, stavi SD card sa MP3 direktorijem za hrvatski jezik
'-------------------------------------------------------------------------------------------------------------
'Koristi Df buffer tako da za vrijeme reprodukcije ne zaustavlja prolaz kroz Do-Loop petlju

'TX = PD.1(hardverski UART) i Busy = PB.1 ; izbor po volji

'Sa $include "dfplayer2.inc" u program ukljuciti slijedece naredbe:

'Time_ms = x : Wait_ms - naredba interaptski izvrsava tocnu pauzu od x ms (zaustavlja Do-Loop)

'Moze se koristiti i 32 bitni brojac Millis koji broji ms od ukljucenja

'Na raspolaganju su i linkovi za brze rutine koje se izvrsavaju u periodi 1ms, 10ms, 100ms i 1s

'Za rad koristi Timer0 prekid, a slobodni su Timer1 i Timer2, treba omoguciti interapte

'Naredbe prije Do-Loop petlje:

'Dfplayer Command , Parameter - naredba slanja komande i opciono parametra na DF player

'najcesca Dfplayer naredba je : Dfplayer Mp3 , x  ; x je broj mp3 zapisa u direktoriju MP3

'Naredbe unutar Do-Loop petlje:

'Dfbuffer_enable - naredba se mora staviti samo jednom na pocetku Do-Loop petlje, a prije
'                  ostalih naredbi za DF player i omogucava pravilan rad Df buffera

'Dfbuffer x  -  naredba za upis u Df buffer ;  x je brojcani naziva mp3 zapisa iz MP3 direktorija

'Dfstop - prekida reprodukciju i brise Df buffer

'Say Lab_message  -  naredba slanja poruke u Df buffer , poruka se nalazi na opcenitoj labeli Lab_message

'Say_number x - izgovor broja iz Single varijable x preko Df buffera (opseg broja je od -9999.9 do 9999.9)

'Say_temperature x - izgovor temperature iz Single varijable x i mjerne jedinice za temperaturu

'Say_day - izgovor dana u tjednju sadrzanog u varijabli Day (1=ponedjeljak...7=nedjelja)

'Say_date - izgovor datuma, mjeseca i godine (hrvatski ili englesni standard izgovora)

'iz Byte varijabli Dte, Month i Year (brojevi u dekatskom obliku)

'Say_time - izgovor vremena HH:MM iz Byte varijabli Hour i Minute (sadrzaj u dekatskom obliku)

'Napomena:Za vrijeme trajanja reprodukcije blokiran je upis novih podataka u Df buffer (Df_flag=1)

'U ovom primjeru je LED = PB.2 za isprobavanje rada programa i za ERROR

'ERROR izlaz za prijavu greske : Buffer overload! (definirano u driveru "dfplayer2.inc")

'Upis u Df buffer radi se u Do-Loop petlji kad je Df_flag=0

'Citanje Df buffera i slanje podataka na DF player izvrsava se u Timer0 interaptu svakih 150ms

'Autor: Mladen Cempuh ; E-mail: cempuh.mladen@gmail.com

'Datum: 26.05.2021.
'****************************************************************************************************************
$regfile = "m328pdef.dat"
'$regfile = "m8def.dat"
$crystal = 16000000                                         ' izabrati ili 8MHz ili 16MHz
$swstack = 50
$hwstack = 60                                               ' VAZNO! iznos od 40 nije dovoljan za HWstack! Preporuka: =>60
$framesize = 64

'---------------------------------------------------------
'DEFINIRANJE ULAZA,IZLAZA I POCETNIH STANJA PINOVA PORTOVA
'DDRn   PORTn
' 0       0   ==> Input 3-state
' 0       1   ==> Input
' 1       0   ==> Output=0
' 1       1   ==> Output=1

  Ddrb = &B11111100                                         ' ulaz PB.0 za tipku KEY1,ulaz PB.1 za Busy, PB.2 izlaz za LED, PB3 izlaz za rasvjetu LCD-a
 Portb = &B11001011                                         ' Izlazi za LCD: DB7=PB.4;DB6=PB.5;DB5=PC.2;DB4=PC.3;E=PC.4;RS=PC.5
  Ddrc = &B11111100                                         ' PC.0 i PC.1 su slobodni ulazi
 Portc = &B00000011                                         '
  Ddrd = &B11100010                                         ' Ulazi za tipke KEY0=PD.0;KEY2=PD.2;KEY3=PD.3;KEY4=PD.4
 Portd = &B11111111                                         ' Izlaz PD.1 za TX hardverskog UART-a
                                                            ' PD5,PD6 i PD7 su slobodni za koristenje (startno postavljeni kao izlazi sa stanjem 1)
'---------------------------------------------------------
'DEFINIRANJE LCD DISPLEJA U 4 BITNOM RADU

Config Lcd = 16 * 2
Config Lcdpin = Pin , Db4 = Portc.3 , Db5 = Portc.2 , Db6 = Portb.5 , Db7 = Portb.4 , Rs = Portc.5 , E = Portc.4
Cls
Cursor Off

'----------------------------------------------------------------------------------------------------------------
Up$ Alias Pind.0                                            ' ulaz za tipku Up (+) za RTC

Down$ Alias Pinb.0                                          ' ulaz za tipku Down (-) za RTC, opcija samo ako je konstanta Down_key=1

Enter$ Alias Pind.2                                         ' ulaz za tipku Enter za RTC

Const Down_key = 1                                          ' Konstanta Down_key=0 --> ne koristi se tipka Down$ ,a ako je 1 koristi se

'------------------------------------------------------------
'           Inicijalizacija I2C komunikacije
'------------------------------------------------------------
 Config Scl = Portd.7                                       ' izlaz SCL

 Config Sda = Portd.6                                       ' izlaz SDA

 I2cinit                                                    ' inicijaliziraj I2C komunikaciju

'-------------------------------------------------------------
$include "DS3231$SE.sub"                                    ' biblioteka za RTC DS3231, preuzeto sa linka Sveta Elektronike

$include "rtc_set.inc"                                      ' biblioteka za namjestanje vremena RTC-a
'--------------------------------------------------------------
 Lcd "ENTER 5s FOR RTC"                                     ' drzi 5s tipku KEY2 (Enter) za ulazak u namjestanje RTC-a

 Wait 2

 Ds3231$write_control &B00000000                            ' SQW=1Hz, Disable INT, Disable Alarm1,Alarm2, Disable Temperature

 Ds3231$set                                                 ' naredba za namjestanje RTC-a

'--------------------------------------------------------------
'Definiraj ulaz za Busy:

Busy Alias Pinb.1

Config Busy = Input                                         ' ulaz za Busy sa DF Playera, 1=slobodan , 0=zauzet

'Definiraj velicinu Df_buffera:

Dfbuffer_len Alias 80                                       'velicina Df_buffera u Words za DF Player(80 Words)

'Definiraj pin za prijavu greske prekoracenja Df_buffera , na taj pin spoji LED za Error

Error Alias Portb.2

$include "dfplayer2.inc"

Language = 1                                                ' 1 = hrvatski , 0 = engleski; izabran je hrvatski
'-------------------------------------------------------------------------------------------------------------
'Definiranje 5 tipki spojenih na PORTD i GND

Key0 Alias Pind.0
Key1 Alias Pinb.0
Key2 Alias Pind.2
Key3 Alias Pind.3
Key4 Alias Pind.4


Led Alias Portb.2

'Config Led = Output                                         ' vec je definirano  kao izlaz sa DDRB
'--------------------------------------------------------------------------------------------------
'  Dfplayer Volume , 31                                      'opciono promjeni glasnocu sa pocetne 28 na 31 --> maksimum
'  Waitms 200                                                'pauza za izvrsenje naredbe

 Enable Interrupts                                          'omoguci sve definirane interapte (potrebno zbog Timer0 interapta)

'-------------------------------------------------------------------------------
' Uputa za koristenje naredbe Wait_ms
'-------------------------------------------------------------------------------
' Naredba: Wait_ms
' za interaptsko odredivanje pauze u ms:
' Primjer koristenja:
'                     Time_ms=1000: Wait_ms  ' pauza 1000ms

'Napomena: Naredba Wait_ms pauzira dalje izvrsavanje naredbi za navedeno
'          vrijeme Time_ms isto kao i naredba Waitms, ali je pauza uvijek tocna
'-------------------------------------------------------------------------------
' Uputa za koristenje 32 bitnog brojaca Millis
'-------------------------------------------------------------------------------

' Brojac broji ms od prorade programa. Podatak je u DWord varijabli Millis.
' Prekoracenje nastaje kroz cca.50 dana kad pocne ponovno od nule.
' Koristi se za izvrsavanje delay funkcije i odlaska na neki dio programa
' nakon isteka vremenskog iznosa u ms, a bez blokiranja izvrsavanja Do-Loop petlje

' Primjer kosistenja:

' Svakih ~500ms izvrsava se potprogram Lab_x, te ne zadrzava prolaza
' kroz Do-Loop petlju

' Dim Previous As Dword , Interval As Word , Difference As Dword

' Interval = 500
' Previous = Millis

' Do

' Difference = Millis - Previous

' If Difference >= Interval Then
'    Gosub Lab_x
'    Previous = Millis
' End If

' Loop

'Lab_x:
'       Return
'-------------------------------------------------------------------------------------------------
' Definiranje niza Single varijabli za probni izgovor njihovih iznosa

  Dim X(10) As Single , I As Byte , Y As Single , W As Word , D As Byte , Predznak As Byte , C As Byte , Sel As Byte

  Dim Room_temp As Single , Temp_read As Word               ' za izracun sobne temperature

  X(1) = -9999.9 : X(2) = 1.8 : X(3) = 1241 : X(4) = 0 : X(5) = -2.5 : X(6) = 2470.1 : X(7) = 8000 : X(8) = -113.2 : X(9) = 505 : X(10) = -1111.7


  Locate 1 , 1 : Lcd "DF Player Ver.2 "

  Locate 2 , 1 : Lcd "Demo primjer 4"

  Wait 2

  Cls

Do

   Dfbuffer_enable                                          ' omoguci rad Df_buffera

   Ds3231$read                                              ' ucitaj sec, min, hour, day, date, month, year

   Locate 1 , 1                                             ' u prvom redu LCD-a od pozicije 1
   Lcd_dan_hr                                               ' ispis dana u tjednu
   Lcd " ";
   Lcd_hh_mm_ss                                             ' i vremena u formatu HH:MM:SS


'U slijedece varijable upisi dekatske iznosi za vrijeme HH:MM, dan u tjednu, datum, mjesec i godinu (a ne BCD)
'Napomena:Ako koristis RTC (npr.DS3231) tada prije upisa BCD iznose pretvori naredbom Makebcd u dekatske iznose


   Hour = Makedec(ds3231$hour)
   Minute = Makedec(ds3231$min)
   Day = Ds3231$day
   Dte = Makedec(ds3231$dte)
   Month = Makedec(ds3231$mth)
   Year = Makedec(ds3231$yr)


   If Df_flag = 0 Then                                      ' ako nema reprodukcije tada ispisi

    Locate 2 , 1 : Lcd "Select(0-9) = " ; Sel ; "  "

   End If

   Debounce Key0 , 0 , Sel_incr , Sub                       ' tipkom Key0 inkrementiraj varijablu Sel

   Debounce Key1 , 0 , Sel_decr , Sub                       ' tipkom Key1 dekrementiraj varijablu Sel

   Debounce Key2 , 0 , Enter , Sub                          ' tipka Key2 je Enter za izvrsenje izabrane opcije sa varijablom Sel


    If Key3 = 0 Then                                        ' ako je pritisnuta tipka Key3 prekini reprodukciju i obrisi Df buffer

      Dfstop

      I = 0                                                 ' postavi indeks za izbor Single varijable X(i) na 0

    End If                                                  ' taj se indeks inkrementira prije izbora ispisa i izgovora Single varijable X(i)

    If Key4 = 0 Then                                        ' ako je pritisnuta tipka Key4 ukljuci Led

      Set Led

   Else

    Reset Led                                               ' u protivnom iskljuci Led

   End If

Loop

'------------------------------------------------------------

Sel_incr:

   Incr Sel

   If Sel > 9 Then Sel = 0                                  ' kruzno inkrementiranje varijable Sel od 0 do 9

Return

Sel_decr:

   Decr Sel

   If Sel = 255 Then Sel = 9                                ' kruzno dekrementiranje varijable Sel od 9 do 0

Return

'-------------------------------------------------------------
Enter:

  Select Case Sel                                           'ovisno o iznosu varijable Sel (Select) 0 do 9 izvrsava:

  Case 0 : Gosub Tko_si _ti                                 'za 0 --> izgovor poruke predstavljanja govornika

  Case 1 : Gosub Say_numbers                                'za 1 --> izgovara brojcani iznos Single varijable X(i)

  Case 2 : Gosub Numbers_1to10                              'za 2 --> izgovara redom brojeve od 1 do 10

  Case 3 : Gosub Numbers_10to1                              'za 3 --> izgovara redom brojeve od 10 do 1

  Case 4 : Gosub Speak_room_temperature                     'za 4 --> izgovara sobnu temperaturu (temperaturu chipa DS3231)

  Case 5 : Gosub Speak_day_time_date                        'za 5 --> izgovara dan,vrijeme,datum,mjesec i godinu iz RTC-a

  Case 6 : Gosub Random_temperature                         'za 6 --> izgovara generirani iznos temperature

  Case 7 : Gosub Gibonni_mp3                                'za 7 --> demo reprodukcija mp3 Gibonni "Vrime da se pomirim sa svitom"

  Case 8 : Gosub Oliver_mp3                                 'za 8 --> demo reprodukcija mp3 Oliver "Dva put san umra" - Instrumental

  Case 9 : Gosub Shallow_mp3                                'za 9 --> demo reprodukcija mp3 "Shallow"

  End Select

Return

'-------------------------------------------------------------
Say_numbers:

   If Df_flag = 1 Then Return                               'ako traje prethodna reprodukcija vrati se

   Incr I                                                   ' inkrementiraj indeks I

   If I > 10 Then I = 1                                     ' ako je nakon inkrementiranja indeks I>10 tada je I=1

   Locate 2 , 1 : Lcd "X(" ; I ; ")=" ; Fusing(x(i) , "####.#") ; "       "       ' ispis Single varijable X(i)

   Say_number X(i)                                          ' izgovor brojcanog iznosa Single varijable X(i)

Return

'-----------------------------------------------------------
Numbers_1to10:

   If Df_flag = 1 Then Return                               'ako traje prethodna reprodukcija vrati se

   Locate 2 , 1 : Lcd "Broji 1 ... 10  "                    ' ispis "Broji 1 ... 10  "

   Say Broji_1do10                                          ' Broji od 1 do 10

  Return

'------------------------------------------------------------
Numbers_10to1:

   If Df_flag = 1 Then Return                               'ako traje prethodna reprodukcija vrati se

   Locate 2 , 1 : Lcd "Broji 10 ... 1  "                    ' ispis "Broji 10 ... 1 "

   Say Broji_10do1                                          ' Broji od 10 do 1

  Return

'------------------------------------------------------------
Gibonni_mp3:

   If Df_flag = 1 Then Return                               ' ako traje prethodna reprodukcija vrati se

   Locate 2 , 1 : Lcd "Muzika: Gibonni "                    ' ispis "Muzika: Gibonni"

  Say Gibonni                                               ' demo reprodukcija mp3 Gibonni "Vrime da se pomirim sa svitom"

  Return

'------------------------------------------------------------
Oliver_mp3:

  If Df_flag = 1 Then Return                                ' ako traje prethodna reprodukcija vrati se

  Locate 2 , 1 : Lcd "Muzika: Oliver  "                     ' ispis "Muzika: Oliver  "

  Say Oliver                                                ' demo reprodukcija mp3 Oliver "Dva put san umra" - Instrumental

Return

'------------------------------------------------------------
Shallow_mp3:

  If Df_flag = 1 Then Return                                ' ako traje prethodna reprodukcija vrati se

  Locate 2 , 1 : Lcd "Muzika: Shallow "                     ' ispis "Muzika: Shallow "

  Say Shallow                                               ' demo reprodukcija mp3 "Shallow"

Return

'------------------------------------------------------------
Tko_si_ti:

 If Df_flag = 1 Then Return                                 ' ako traje prethodna reprodukcija vrati se

  Locate 2 , 1 : Lcd "Tko si ti ?     "                     ' ispis "Tko si ti ?     "

  Say Ja_sam                                                ' Kaze : "Ja sam ...."

Return

'------------------------------------------------------------
Random_temperature:

' Generiranje nekog broja Y za iznos temperature

 If Df_flag = 1 Then Return                                 'ako traje prethodna reprodukcija vrati se

    W = Rnd(99) : D = Rnd(9)                                'generiraj iznos temperature u Single varijabli Y
    Predznak = Rnd(9)
    C = Rnd(9)
    Y = 0
    If C >= 5 Then
    Y = 0.1 * D
    End If

    Y = Y + W
    If Predznak <= 5 Then
    Y = -1 * Y
    End If

    Locate 2 , 1 : Lcd "Temp. = " ; Fusing(y , "##.#") ; Chr(223) ; "C   "       ' za ispis random temperature Y

    Say_temperature Y                                       'reci generiran broj za temperaturu i reci ispravno jedinicu za Celzij

Return
'-----------------------------------------------------------
Speak_day_time_date:

  If Df_flag = 1 Then Return                                'ako traje prethodna reprodukcija vrati se

   Locate 2 , 1

   Lcd "                "                                   ' obrisi drugi red LCD-a

   Locate 2 , 1

   Lcd_dd_mm_20yy                                           ' ispis datuma, mjeseca i godine

   Say Today_is                                             ' kaze "Danas je" ili "Today is"

   Say_day                                                  ' reci koji je dan

   Say Pause_1s                                             ' tisina 1s

   Say The_time_is                                          ' kaze "Sada je" ili "The time is"

   Say_time                                                 ' reci vrijeme HH:MM

   Say Pause_1s                                             ' tisina 1s

   Say Date_is                                              ' kaze "Datum je" ili "Date is"

   Say_date                                                 ' reci datum , mjesec i godinu (EU nacin za hrvatski ili UK/US nacin za engleski)

Return
'-----------------------------------------------------------
Speak_room_temperature:

 If Df_flag = 1 Then Return                                 'ako traje prethodna reprodukcija vrati se

 Ds3231$read_temp Temp_read                                 'ucitaj temperaturu iz chipa DS3231

 Room_temp = Temp_read / 256                                'izracunaj temperaturu u stupnjevima Celzija

 Locate 2 , 1 : Lcd "T(sobe) = " ; Fusing(room_temp , "##.#") ; Chr(223) ; "C "       ' ispis sobne temperature (chipa DS3231)

 Say Temperature_is

 Say_temperature Room_temp                                  ' kaze pozitivnu temperaturu iz DS3231 RTC chipa (jedna decimala)

Return

'***************************************************************************

'---------------------------------------------------------------------------
'       Link za dodavanje brzih rutina koje se izvrsavaju svaku 1ms
'---------------------------------------------------------------------------
Every_1ms:


   Return
'---------------------------------------------------------------------------
'      Link za dodavanje brzih rutina koje se izvrsavaju svakih  10ms
'---------------------------------------------------------------------------
Every_10ms:


   Return
'---------------------------------------------------------------------------
'      Link za dodavanje brzih rutina koje se izvrsavaju svakih 100ms
'---------------------------------------------------------------------------
Every_100ms:


   Return
'---------------------------------------------------------------------------
'      Link za dodavanje brzih rutina koje se izvrsavaju svake 1s
'---------------------------------------------------------------------------
Every_1s:

    Toggle Portd.5                                          ' invertiraj izlaz PD.5 svake sekunde za primjer koristenja linka Every_1s

   Return
'---------------------------------------------------------------------------
End                                                         ' Kraj programa

$include "MP3.bas"                                          'MP3.bas datoteka u programu koristenih mp3 zapisa